//
//  lkf2023Tests.swift
//  lkfcTests
//
//  Created by Alexander Holzer on 25.07.2022.
//  Copyright © 2022 Alexander Holzer. All rights reserved.
//

import XCTest
@_implementationOnly import LibLKF
@testable import lkfc

class lkf2023Tests: XCTestCase {

    func createContextWith (model: UInt32, year: Int32? = nil, lines : [String]) -> LKFContext {
        let c = lkfcTests.createContextWith(model:model, lines:lines)
        if let year {
            c.setContextParameters(year: year)
        }
        return c
    }
    
    func combine (_ first:[String], _ second:[String]) -> [String] {
        return lkfcTests.combine(first, second)
    }

    override func setUpWithError() throws {
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }

    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }
    
    func expect(error: String, in list:[String]) {
        XCTAssert(list.contains(where: {$0.contains(error)}), "\(error) expected")
    }
    
    static var sapsTest = [
        "X01|K427|1234||20220102|1443|S|19310501|85|M|AT|AT|4941|41214|A|A||||E|20220112|1612|UB|N|||",
        "X02|K427|1234||20220102|001|11111100|20220102|1443|||||||N|85||00001",
        "X02|K427|1234||20220102|002|12718100|20220105|0851||20220106|1610|1|||N|85||00002",
        "X02|K427|1234||20220102|003|11211100|20220106|1610|||||||N|85||00003",
        "X03|K427|1234||20220102|F10.0|H|||||00001",
    "I11|K427|1234||20220102|002|20220105|00|1|3|0|0|00|0|0|0|0|0|0|1|6.71|7.4|0259|32|1||4|4|044|000000000000000000000|36.5|1|00199|050|0.3|0.73|170|001|5",
        "I12|K427|1234||20220102|002|20220105|1|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0",
        "I12|K427|1234||20220102|002|20220106|1|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0",
        "L01|2022|K427|12718100|I|E1|"
    ]
    
    func testFormatIntensiv() throws {
        var context = createContextWith(model: 2022, lines: lkf2023Tests.sapsTest)
        _ = context.execute()
        let res = context.getResultLine(forIndex: 0)
        XCTAssert(res == "X05|K427|1234||20220102|HDG20.08|D|44|633|||1758||||2435", "result differs")
        
        context = createContextWith(model: 2023, lines: lkf2023Tests.sapsTest)
        context.setContextParameters(year: 2022)
        _ = context.execute()
        
        let errors = context.getErrorsAndWarnings()
        expect(error: "EI2G", in: errors)
        expect(error: "EI2H", in: errors)
        expect(error: "EI2P", in: errors)
        expect(error: "EI2V", in: errors)
        expect(error: "EI2W", in: errors)
    }

    static var chrono = [
        "X01|K714|1234||20190621|1200|S|19360501|80|M|AT|AT|1010|90101|A|P||||E|20190625|1200|11|J|0",
        "X02|K714|1234||20190621|001|11111100|20190621||J|20190623|||||N|80|",
        "X02|K714|1234||20190621|002|11111101|20190624||J||||||N|80|",
        "X03|K714|1234||20190621|R53|H|D||",
        "X04|K714|1234||20190621|BG020||J|0001|20190621|2024|18111600|"]

    static var chrono_accept = [
        "X06|K714|1234||20190621|001|WX4H|accept 2022"]
    
    func testWX4HAccept() throws {
        var context = createContextWith(model: 2022, lines: lkf2023Tests.chrono)
        _ = context.execute()
        //let res = context.getResultLine(forIndex: 0)
        var errors = context.errorsWith (code:"WX4H") // Verlegungen nicht aufeinanderfolgend
        XCTAssert(errors != nil, "expected WX4H")
        
        let chronoaccept = combine(lkf2023Tests.chrono, lkf2023Tests.chrono_accept)
        
        context = createContextWith(model: 2022, lines: chronoaccept)
        _ = context.execute()
        errors = context.errorsWith (code:"WX4H") // Verlegungen nicht aufeinanderfolgend
        XCTAssert(errors == nil, "accepted WX4H not expected in 2022")
        
        context = createContextWith(model: 2023, lines: chronoaccept)
        _ = context.execute()
        errors = context.errorsWith (code:"WX4H") // Verlegungen nicht aufeinanderfolgend
        XCTAssert(errors != nil, "WX4H expected, cannot be accepted in 2023")
    }

    static var fall_unterbrent = [
        "X01|K714|1234||20190121|1200|S|19960601|80|M|AT|AT|1010|90101|A|P||||E|20190624|1200|11|J|0",
        "X02|K714|1234||20190121|01|11111100|20190121||J||||||N|80|",
        "X02|K714|1234||20190121|02|10000000|20190126||J||||||N|80|",
        "X02|K714|1234||20190121|03|11111100|20190130||J||||||N|80|",
        "X02|K714|1234||20190121|04|10000000|20190622||J||||||N|80|",
        "X03|K714|1234||20190121|B21.3|H|D||",
        "X04|K714|1234||20190121|XA160||J|0001|20190121|2024|18111600|"]

    func testUnterBr() throws {
        var context = createContextWith(model: 2021, year: 2019, lines: lkf2023Tests.fall_unterbrent)
        _ = context.execute()
        var errors = context.errorsWith (code:"EX50") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors == nil, "accepted EX50 not expected in 2021")
        
        context = createContextWith(model: 2022, year: 2019, lines: lkf2023Tests.fall_unterbrent)
        _ = context.execute()
        errors = context.errorsWith (code:"EX50") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors != nil, "accepted EX50 expected in 2022")
        
        context = createContextWith(model: 2023, year: 2019, lines: lkf2023Tests.fall_unterbrent)
        _ = context.execute()
        errors = context.errorsWith (code:"EX50") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors != nil, "accepted EX50 expected in 2023")
    }
    
    static var fall_unterbrP = [
        "X01|K901|1234||20220121|1200|S|19960601|80|M|AT|AT|1010|90101|A|P||||E|20220624|1200|11|J|0",
        "X02|K901|1234||20220121|01|11111100|20220121||J||||||N|80|",
        "X02|K901|1234||20220121|02|10000000|20220126||J||||||N|80|",
        "X02|K901|1234||20220121|03|11111100|20220322||J||||||N|80|",
        "X02|K901|1234||20220121|04|10000000|20220622||J||||||N|80|",
        "X02|K901|1234||20220121|05|11111100|20220624||J||||||N|80|",
        "X03|K901|1234||20220121|B21.3|H|D||",
        "X04|K901|1234||20220121|XA160||J|0001|20220121|2024|18111600|"]

    func testUnterBrPrikraf() throws {
        var context = createContextWith(model: 2022, year: 2022, lines: lkf2023Tests.fall_unterbrP)
        _ = context.execute()
        var errors = context.errorsWith (code:"EX32") // Unterbrecher 5 Tage
        XCTAssert(errors != nil, "EX32 expected in 2022")
        errors = context.errorsWith (code:"EX34") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors != nil, "EX34 expected in 2022")
        
        let prikraf = lkfcFeatureTests.replace(source: lkf2023Tests.fall_unterbrP, from: "K901", to: "K911")
        context = createContextWith(model: 2022, year: 2022, lines: prikraf)
        _ = context.execute()
        errors = context.errorsWith (code:"EX32") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors == nil, "EX32 not expected for P-KAs in 2022")
        errors = context.errorsWith (code:"EX34")
        XCTAssert(errors == nil, "EX34 not expected for P-KAs in 2022")
        
        context = createContextWith(model: 2021, year: 2022, lines: prikraf)
        _ = context.execute()
        errors = context.errorsWith (code:"EX32") // Unterbrecher als Entlassungsabteilung ungueltig
        XCTAssert(errors != nil, "EX32 expected for P-KAs in 2021")
        errors = context.errorsWith (code:"EX34")
        XCTAssert(errors != nil, "EX34 expected for P-KAs in 2021")
    }
    
    static var selteneErkr = [ // Zusatzdiagnose "14" -> muss in DIAGLIST enthalten sein
        "X01|K952|123||20220501|0727|S|20130101|1|M|SY|AT|1150|91501|K|P||||E|20220501|1900|11|J|2||",
        "X02|K952|123||20220501|001|14518501|20220501|0727||||||14518501|N|1|",
        "X02|K952|123||20220501|002|12518101|20220501|0944||||||12518101|N|1|",
        "X02|K952|123||20220501|003|12518102|20220501|1533||||||12518102|N|1|",
        "X03|K952|123||20220501|C74.1|H|||",
        "X03|K952|123||20220501|Z51.8|Z|||",
        "X03|K952|123||20220501|14|Z|||",
        "X04|K952|123||20220501|DH010||J|1|20220501|1114|12518101||",
        "X04|K952|123||20220501|DL010||J|1|20220501|1114|12518101||",
        "X04|K952|123||20220501|FZ040||J|1|20220501|1114|12518101||",
        "X04|K952|123||20220501|XA230||J|1|20220501|1114|12518101||",
        "X04|K952|123||20220501|ZZ030||J|1|20220501|1114|12518101||"
    ]
    
    static var fall_physanw =
    [
        "X01|K706|345||20230121|0800|I|19710101|45|W|AT|AT|6410|70357|||||||||18|J|2",
        "X02|K706|345||20230121|001|16111300|20230121|0800|J||||||N|55|123",
        "X04|K706|345||20230121|ZZ533||J|1|20230121|0800|16111300||123",
        "X04|K706|345||20230121|AN522||J|1|20230121|0800|16111300||123",
        "X04|K706|345||20230121|ZD510||J|1|20230121|0800|16111300||12"
    ]

    static var fall_physanwJN =
    [
        "X01|K706|345||20230121|0800|I|19710101|45|W|AT|AT|6410|70357|||||||||18|J|2",
        "X02|K706|345||20230121|001|16111300|20230121|0800|J||||||N|55|123",
        "X02|K706|345||20230121|002|16111300|20230121|0800|N||||||N|55|123",
        "X04|K706|345||20230121|ZZ533||J|1|20230121|0800|16111300||123",
        "X04|K706|345||20230121|AN522||J|1|20230121|0800|16111300||123",
        "X04|K706|345||20230121|ZD510||J|1|20230121|0800|16111300||12"
    ]

    
    func testEX4X() throws {
        var context = createContextWith(model: 2023, year: 2023, lines: lkf2023Tests.fall_physanw)
        _ = context.execute()
        var errors = context.errorsWith (code:"EX4X") // Unterbrecher 5 Tage
        XCTAssert(errors != nil, "EX4X expected in 2023")
        context = createContextWith(model: 2023, year: 2023, lines: lkf2023Tests.fall_physanwJN)
        _ = context.execute()
        errors = context.errorsWith (code:"EX4X") // Unterbrecher 5 Tage
        XCTAssert(errors == nil, "EX4X not expected in 2023")
        
        context = createContextWith(model: 2022, year: 2023, lines: lkf2023Tests.fall_physanwJN)
        _ = context.execute()
        errors = context.errorsWith (code:"EX4X") // Unterbrecher 5 Tage
        XCTAssert(errors != nil, "EX4X expected in 2022")
        //errors = context.errorsWith (code:"EX34") // Unterbrecher als Entlassungsabteilung ungueltig
        //XCTAssert(errors != nil, "EX34 expected in 2022")
    }

}
